__all__ = 'ImportExistingRemoteProjectWidget'

from PySide2 import QtCore
from PySide2 import QtWidgets

import gm_meta_plugin
from gm_meta_plugin.gridmarkets import constants as api_constants
from gm_meta_plugin import attribute_types

from gm_qt_plugin import qt_constants
from gm_qt_plugin import qt_utils
from gm_qt_plugin.qt_plugin_accessor import QtPluginAccessor
from gm_qt_plugin.widgets.qt_project_attribute_instance import QtProjectAttributeInstance


class ImportExistingRemoteProjectWidget(QtWidgets.QWidget, QtPluginAccessor):

    def __init__(self, plugin, parent=None):
        QtWidgets.QWidget.__init__(self, parent=parent)
        QtPluginAccessor.__init__(self, plugin)

        user_interface = plugin.get_user_interface()

        """ Create Widgets """
        self.prompt_label = QtWidgets.QLabel("To use this option you must have already uploaded a project to Envoy. " +
                                             "Check for existing projects using the button below.")

        self.prompt_btn = QtWidgets.QPushButton("Check for Existing Projects")
        self.prompt_btn.setIcon(qt_constants.NEW_FILE_ICON_WHITE)
        self.prompt_btn.clicked.connect(
            lambda: user_interface.refresh_remote_root_dirs(self.refresh_remote_root_dirs_slot))

        cached_api_schema = plugin.get_api_client().get_cached_api_schema()

        self.project_attribute_widgets = cached_api_schema.create_project_attribute_widgets()

        for key, widget in self.project_attribute_widgets.iteritems():
            if hasattr(widget, 'input_value_changed'):
                widget.input_value_changed.connect(self.redraw)

        """ Create Layouts """

        self.group_box_1 = QtWidgets.QGroupBox()
        self.group_box_2 = None

        self.refresh_layout = QtWidgets.QVBoxLayout()
        self.refresh_layout.setAlignment(QtCore.Qt.AlignHCenter)
        self.refresh_layout.addWidget(self.prompt_btn)
        self.refresh_layout.addWidget(self.prompt_label)

        self.group_box_1.setLayout(self.refresh_layout)
        self.widget = QtWidgets.QWidget()

        self.stacked_layout = QtWidgets.QStackedLayout(self)

        self.refresh_layout_index = self.stacked_layout.addWidget(self.group_box_1)
        self.dynamic_layout_index = self.stacked_layout.addWidget(self.widget)

    def redraw(self):

        # hide all old widgets
        while self.form_layout.rowCount():
            row = self.form_layout.takeAt(0)

            if row is None:
                break

            widget = row.widget()
            widget.hide()

        root = self.get_plugin().get_api_client().get_cached_api_schema().get_project_definition_schema().get_root_project_attribute()
        enabled = not QtProjectAttributeInstance.draw_project_attributes(self.form_layout,
                                                                         root,
                                                                         self.project_attribute_widgets,
                                                                         remote_source=True)

        self.submit_btn.setEnabled(enabled)

    def refresh_remote_root_dirs_slot(self, result):

        if self.get_plugin().get_api_client().get_cached_root_directories():
            self.stacked_layout.setCurrentIndex(self.dynamic_layout_index)

            dynamic_widget = self.stacked_layout.currentWidget()
            self.layout = QtWidgets.QVBoxLayout()
            self.layout.setContentsMargins(0, 0, 0, 0)
            self.form_layout = QtWidgets.QFormLayout()

            self.submit_btn = QtWidgets.QPushButton("Import Remote Project")
            self.submit_btn.clicked.connect(self.execute)
            self.dynamic_layout = QtWidgets.QVBoxLayout()
            self.dynamic_layout.addLayout(self.form_layout)
            self.dynamic_layout.addWidget(self.submit_btn)
            self.group_box_2 = QtWidgets.QGroupBox()
            self.group_box_2.setLayout(self.dynamic_layout)
            self.layout.addWidget(self.group_box_2)
            dynamic_widget.setLayout(self.layout)

            self.redraw()
        else:
            self.stacked_layout.setCurrentIndex(self.refresh_layout_index)

    def execute(self):
        plugin = self.get_plugin()
        api_client = plugin.get_api_client()
        api_schema = api_client.get_cached_api_schema()
        logger = plugin.get_logging_coordinator().get_logger(__name__)
        root = api_schema.get_project_definition_schema().get_root_project_attribute()

        project_name = root.get_value(widgets=self.project_attribute_widgets, is_remote=True)

        # check the project exists
        if project_name not in api_client.get_root_directories(ignore_cache=True):
            # self.report({'ERROR'}, "Project '" + project_name + "' no longer exists.")
            logger.error("Project '" + project_name + "' no longer exists.")
            return

        # get the files for this project
        envoy_files = api_client.get_remote_project_files(project_name)
        files = list(map(lambda x: x.get_key(), envoy_files))

        def _do_file_paths_exist(project_attribute):

            attribute = project_attribute.get_attribute()

            if attribute.get_type() == gm_meta_plugin.AttributeType.NULL:
                return True

            value = project_attribute.get_value(widgets=self.project_attribute_widgets, is_remote=True)

            # check file paths exist
            if attribute.get_type() == gm_meta_plugin.AttributeType.STRING and \
                    attribute.get_subtype() == attribute_types.StringSubtype.PATH.value and \
                    attribute.get_subtype_kwargs().get(
                        api_constants.SUBTYPE_KEYS.STRING.PATH.FILE_MODE) == api_constants.SUBTYPE_KEYS.STRING.PATH.FILE_PATH:

                if value not in files:
                    logger.error("File path '" + str(value) + "' does not exist under project '" +
                                 project_name + "'.")
                    return False

            return _do_file_paths_exist(project_attribute.transition(value))

        # check files actually exist on the remote project
        if not _do_file_paths_exist(root):
            return

        try:
            attributes = qt_utils.get_project_attributes(plugin, self.project_attribute_widgets, is_remote=True)
            remote_project = gm_meta_plugin.gridmarkets.RemoteProject(attributes.get(api_constants.API_KEYS.PROJECT_NAME),
                                                                      attributes.get(api_constants.API_KEYS.APP),
                                                                      set(),
                                                                      attributes,
                                                                      api_client)

            plugin.get_remote_project_container().append(remote_project)

            # switch to the projects layout
            main_window = self.get_plugin().get_user_interface().get_main_window()
            main_window.get_main_ui_widget().get_main_ui_switcher().display_remote_project_container()

        except ValueError as e:
            pass
        except gm_meta_plugin.errors.RejectedTransitionInputError as e:
            pass
