__all__ = 'Job Preset'

import gm_meta_plugin
from gm_meta_plugin.gridmarkets import constants as api_constants
from gm_qt_plugin.widgets.job_preset_attribute import JobPresetAttribute

from gm_qt_plugin import qt_constants, qt_utils
from PySide2 import QtCore
from PySide2 import QtWidgets

from gm_qt_plugin.widgets.qhline import QHLine
from gm_qt_plugin.widgets.input_widgets.check_box import CheckBox
from gm_qt_plugin.qt_plugin_accessor import QtPluginAccessor


class JobPresetAttributeGroup(QtWidgets.QWidget, QtPluginAccessor):

    def __init__(self, plugin, job_preset, name, parent=None):
        QtWidgets.QWidget.__init__(self, parent=parent)
        QtPluginAccessor.__init__(self, plugin)

        self._job_preset = job_preset
        self._name = name

        ''' Create Widgets '''
        self._header = QtWidgets.QLabel(name)
        self._header.setProperty('h4', True)

        self._separator = QHLine()

        self._attribute_group_box = QtWidgets.QGroupBox()
        self._attribute_group_box.setAlignment(QtCore.Qt.AlignTop)

        ''' Create Layouts '''
        self._layout = QtWidgets.QVBoxLayout(self)
        self._layout.setContentsMargins(0,0,0,0)

        show_hidden_job_preset_attributes = JobPreset.show_hidden_job_preset_attributes()
        job_preset_attributes = self.get_job_preset_attributes()

        self._grid_layout = QtWidgets.QGridLayout()
        self._grid_layout.setAlignment(QtCore.Qt.AlignTop)
        self._grid_layout.setSpacing(8)
        self._grid_layout.setContentsMargins(0, 0, 0, 0)

        col2 = 2 if show_hidden_job_preset_attributes else 1
        col3 = 3 if show_hidden_job_preset_attributes else 2

        # add widgets
        current_row = 0
        for i, job_preset_attribute in enumerate(job_preset_attributes):

            job_attribute = job_preset_attribute.get_job_attribute()

            # If hidden job preset attributes are not visible
            if not show_hidden_job_preset_attributes:

                # Then hide attributes which only have one inference source and it is either constant or project defined
                # We hide these since they are not modifiable by the user so they can be hidden.
                if len(job_attribute.get_inference_sources()) == 1:
                    if job_preset_attribute.get_inference_source() == gm_meta_plugin.InferenceSource.get_constant_inference_source():
                        continue

                    if job_preset_attribute.get_inference_source() == gm_meta_plugin.InferenceSource.get_project_inference_source():
                        continue

            current_row = current_row + 1

            # display name
            self._grid_layout.addWidget(job_preset_attribute.get_display_name_label(), current_row, 0)

            # key
            if show_hidden_job_preset_attributes:
                self._grid_layout.addWidget(job_preset_attribute.get_key_label(), current_row, 1)

            # main widget
            job_preset_attribute.init()
            self._grid_layout.addWidget(job_preset_attribute, current_row, col2)

            # inference source checkbox
            self._grid_layout.addWidget(job_preset_attribute.get_inference_source_widget(), current_row, col3)

        # set the column stretch factors
        self._grid_layout.setColumnStretch(0, 0)
        if show_hidden_job_preset_attributes:
            self._grid_layout.setColumnStretch(1, 0)
        self._grid_layout.setColumnStretch(col2, 3)
        self._grid_layout.setColumnStretch(col3, 0)

        group_box_v_layout = QtWidgets.QVBoxLayout()
        group_box_v_layout.setAlignment(QtCore.Qt.AlignTop)
        group_box_v_layout.setContentsMargins(10, 7, 10, 7)
        group_box_v_layout.addWidget(self._header)
        group_box_v_layout.addWidget(self._separator)
        group_box_v_layout.addSpacing(10)
        group_box_v_layout.addLayout(self._grid_layout)

        self._attribute_group_box.setLayout(group_box_v_layout)
        self._layout.addWidget(self._attribute_group_box)

    def get_job_preset(self):
        return self._job_preset

    def get_name(self):
        return self._name

    def get_job_preset_attributes(self):
        return self.get_job_preset().get_job_preset_attributes_in_group(self.get_name())


class JobPreset(QtWidgets.QWidget, gm_meta_plugin.JobPreset, QtPluginAccessor):
    _show_hidden_job_preset_attributes = CheckBox()
    _show_hidden_job_preset_attributes_label = QtWidgets.QLabel("Show Hidden Job Preset Attributes")

    def __init__(self, plugin, name, id, job_definition, toggle_lock_callback, is_locked=False, parent=None):

        self._id = id
        job_preset_attributes = list(map(lambda job_attribute: JobPresetAttribute(plugin, self, job_attribute),
                                         job_definition.get_attributes()))

        QtWidgets.QWidget.__init__(self, parent=parent)
        gm_meta_plugin.JobPreset.__init__(self, name, id, job_definition, job_preset_attributes, is_locked=is_locked)
        QtPluginAccessor.__init__(self, plugin)

        self._toggle_lock_callback = toggle_lock_callback
        self._column_count = 1

        ''' Create Widgets '''
        self._job_preset_attribute_group_widgets = []
        for attribute_group_index, attribute_group in enumerate(self.get_job_definition().get_attribute_groups()):
            self._job_preset_attribute_group_widgets.append(JobPresetAttributeGroup(self.get_plugin(),
                                                                                    self,
                                                                                    attribute_group))

        ''' Create Layouts '''
        self._layout = QtWidgets.QHBoxLayout(self)
        self._layout.setContentsMargins(0,0,0,0)
        self._layout.setSpacing(qt_constants.MARGIN_SIZE)
        self._column_widgets = []
        self._redraw_columns()

    def _redraw_columns(self):
        # create column widgets
        # hide all existing column widgets
        for column_widget in self._column_widgets:
            column_widget.hide()

        for i in range(self._column_count):
            # if the column_widget does not already exist then create one
            if len(self._column_widgets) < i + 1:
                column_widget = QtWidgets.QWidget()

                column_layout = QtWidgets.QVBoxLayout()
                column_layout.setContentsMargins(0, 0, 0, 0)
                column_layout.setAlignment(QtCore.Qt.AlignTop)
                column_layout.setSpacing(qt_constants.MARGIN_SIZE / 2)

                # add a stretch that all other widgets should be inserted after
                column_layout.addStretch()

                # set direction to bottom to top so the stretch stays at the bottom and all groups can be easily added
                # "before" the stretch with 'addWidget'
                column_layout.setDirection(QtWidgets.QBoxLayout.BottomToTop)

                column_widget.setLayout(column_layout)
                self._column_widgets.append(column_widget)

            self._layout.addWidget(self._column_widgets[i])
            self._column_widgets[i].show()

        job_preset_attribute_group_count = len(self.get_job_definition().get_attribute_groups())
        groups_per_column = job_preset_attribute_group_count / self._column_count
        current_column = 0

        for index, jp_attr_group_widget in enumerate(self._job_preset_attribute_group_widgets):
            if (index - (
                    current_column * groups_per_column)) / groups_per_column >= 1 and current_column < self._column_count - 1:
                current_column = current_column + 1

            self._column_widgets[current_column].layout().insertWidget(1, jp_attr_group_widget)

    def update_app_inferred_values(self):
        for job_preset_attribute in self.get_job_preset_attributes():
            job_preset_attribute.update_app_inferred_values()

    def toggle_locked_state(self):
        gm_meta_plugin.JobPreset.toggle_locked_state(self)
        self._toggle_lock_callback(self)

    def set_locked_state(self, is_locked):
        gm_meta_plugin.JobPreset.set_locked_state(self, is_locked)

    def show_hidden_job_preset_attributes_slot(self, state):
        pass

    def get_icon(self):
        job_definition = self.get_job_definition()
        job_definition_app_attribute = job_definition.get_attribute_with_key(api_constants.API_KEYS.APP)
        job_definition_app = job_definition_app_attribute.get_attribute().get_default_value()
        return qt_utils.get_product_icon(job_definition_app)

    def get_columns(self):
        if self.size().width() < 1250:
            return 1
        else:
            return 2

    def resizeEvent(self, event):
        # If old_width is not -1:
        new_column_count = self.get_columns()

        if self._column_count != new_column_count:
            self._column_count = new_column_count
            self._redraw_columns()

    @staticmethod
    def show_hidden_job_preset_attributes():
        return JobPreset._show_hidden_job_preset_attributes.checkState() == QtCore.Qt.Checked
