

__all__ = 'LogHistoryContainer'

from PySide2 import QtCore
from PySide2 import QtGui
from PySide2 import QtWidgets

import gm_meta_plugin
from gm_qt_plugin import qt_constants, qt_utils
from gm_qt_plugin.widgets.right_aligned_qmenu import RightAlignedQMenu


class LogHistoryContainer(QtWidgets.QWidget, gm_meta_plugin.LogHistoryContainer):
    _LOGGING_INFO_SPACER = u'   '
    INITAL_BUFFER_SIZE = 100

    append_signal = QtCore.Signal(object)
    redraw_signal = QtCore.Signal()

    def __init__(self, plugin, log_items, parent=None):
        QtWidgets.QWidget.__init__(self, parent=parent)
        gm_meta_plugin.LogHistoryContainer.__init__(self, plugin, log_items)

        """ Create Widgets """
        self.expand_toggle_btn = QtWidgets.QPushButton()

        self._header_label = QtWidgets.QPushButton("Logging Console")
        self._header_label.setFlat(True)
        self._header_label.setIcon(qt_constants.CONSOLE_ICON_WHITE)

        self._logging_output = QtWidgets.QTextEdit()
        self._logging_output.setFrameShape(QtWidgets.QFrame.NoFrame)
        self._logging_output.setProperty(qt_constants.STYLE_PROPS.CONSOLE, True)
        self._logging_output.setLineWrapMode(QtWidgets.QTextEdit.WidgetWidth)
        self._logging_output.setReadOnly(True)

        self._toggle_show_log_dates_action = QtWidgets.QAction(u"Show dates", self)
        self._toggle_show_log_times_action = QtWidgets.QAction(u"Show times", self)
        self._toggle_show_logger_names_action = QtWidgets.QAction(u"Show logger names", self)
        self._toggle_show_log_levels_action = QtWidgets.QAction(u"Show levels", self)

        self._toggle_show_log_dates_action.setCheckable(True)
        self._toggle_show_log_times_action.setCheckable(True)
        self._toggle_show_log_times_action.setChecked(True)
        self._toggle_show_logger_names_action.setCheckable(True)
        self._toggle_show_log_levels_action.setCheckable(True)

        self._clear_logs_btn = QtWidgets.QPushButton(qt_constants.TRASH_ICON_WHITE, "")
        self._clear_logs_btn.setFlat(True)

        self._copy_logs_to_clipboard_btn = QtWidgets.QPushButton(qt_constants.COPY_ICON_WHITE, "")
        self._copy_logs_to_clipboard_btn.setFlat(True)

        self._save_logs_to_file_btn = QtWidgets.QPushButton(qt_constants.NEW_FILE_ICON_WHITE, "")
        self._save_logs_to_file_btn.setFlat(True)

        self._display_options_menu_btn = QtWidgets.QPushButton("Display Options")
        self._display_options_menu_btn.setFixedWidth(150)
        self._display_options_menu_btn.setIcon(qt_constants.MENU_ICON_WHITE)

        self._display_options_menu = RightAlignedQMenu("Display Options", parent=self._display_options_menu_btn)
        self._display_options_menu.addSection("Display Options")
        self._display_options_menu.addAction(self._toggle_show_log_dates_action)
        self._display_options_menu.addAction(self._toggle_show_log_times_action)
        self._display_options_menu.addAction(self._toggle_show_logger_names_action)
        self._display_options_menu.addAction(self._toggle_show_log_levels_action)
        self._display_options_menu_btn.setMenu(self._display_options_menu)
        self._display_options_menu_btn.setFlat(True)

        """ Create Connections """
        self.append_signal.connect(self._append_log_item_to_output, type=QtCore.Qt.QueuedConnection)
        self.redraw_signal.connect(self._redraw_logs, type=QtCore.Qt.QueuedConnection)

        self._toggle_show_log_dates_action.triggered.connect(self._redraw_logs)
        self._toggle_show_log_times_action.triggered.connect(self._redraw_logs)
        self._toggle_show_logger_names_action.triggered.connect(self._redraw_logs)
        self._toggle_show_log_levels_action.triggered.connect(self._redraw_logs)

        self._clear_logs_btn.clicked.connect(self.reset)
        self._copy_logs_to_clipboard_btn.clicked.connect(self._copy_logs_to_clipboard_slot)
        self._save_logs_to_file_btn.clicked.connect(lambda: qt_utils.save_logs_to_file(plugin))

        """ Create Layouts """
        self._header_layout = QtWidgets.QHBoxLayout()
        self._header_layout.setAlignment(QtCore.Qt.AlignLeft)
        self._header_layout.addWidget(self._header_label)

        self._header_controls_layout = QtWidgets.QHBoxLayout()
        self._header_controls_layout.setContentsMargins(qt_constants.MARGIN_SIZE, 0, qt_constants.MARGIN_SIZE, 0)
        self._header_controls_layout.addLayout(self._header_layout)
        self._header_controls_layout.addStretch()
        self._header_controls_layout.addWidget(self._display_options_menu_btn)

        self._logging_controls = QtWidgets.QVBoxLayout()
        self._logging_controls.setAlignment(QtCore.Qt.AlignTop)
        self._logging_controls.addWidget(self._copy_logs_to_clipboard_btn)
        self._logging_controls.addWidget(self._save_logs_to_file_btn)
        self._logging_controls.addStretch()
        self._logging_controls.addWidget(self._clear_logs_btn)

        self.layout = QtWidgets.QHBoxLayout(self)
        self.layout.addWidget(self._logging_output)
        self.layout.addLayout(self._logging_controls)

    def _copy_logs_to_clipboard_slot(self):
        self.get_plugin().get_logging_coordinator().copy_logs_to_clipboard()

    def get_header_layout(self):
        return self._header_controls_layout

    def show_header_controls(self):
        self._display_options_menu_btn.show()

    def hide_header_controls(self):
        self._display_options_menu_btn.hide()

    def show_log_dates(self):
        return self._toggle_show_log_dates_action.isChecked()

    def show_log_times(self):
        return self._toggle_show_log_times_action.isChecked()

    def show_logger_names(self):
        return self._toggle_show_logger_names_action.isChecked()

    def show_log_levels(self):
        return self._toggle_show_log_levels_action.isChecked()

    def _redraw_logs(self):
        self._logging_output.clear()

        buffered_items = self.get_all()[-LogHistoryContainer.INITAL_BUFFER_SIZE:]

        for log_item in buffered_items:
            self._append_log_item_to_output(log_item)

    def _append_log_item_to_output(self, log_item):

        show_date = self.show_log_dates()
        show_time = self.show_log_times()
        show_name = self.show_logger_names()
        show_level = self.show_log_levels()

        # line text
        text = u''

        if show_date:
            text += log_item.get_date() + self._LOGGING_INFO_SPACER

        if show_time:
            text += log_item.get_time() + self._LOGGING_INFO_SPACER

        if show_name:
            text += log_item.get_logger_name() + self._LOGGING_INFO_SPACER

        if show_level:
            text += log_item.get_level() + self._LOGGING_INFO_SPACER

        text += log_item.get_message()

        # Set text color depending on severity
        if log_item.get_level() == 'ERROR':
            self._logging_output.setTextColor(QtGui.QColor(255,0,0))
        else:
            self._logging_output.setTextColor(QtGui.QColor(168, 168, 168))

        self._logging_output.append(text)

        # scroll to bottom
        self._logging_output.verticalScrollBar().setValue(self._logging_output.verticalScrollBar().maximum())

    def append(self, item, focus_new_item=True):
        gm_meta_plugin.LogHistoryContainer.append(self, item, focus_new_item=focus_new_item)
        self.redraw_signal.emit()

    def reset(self):
        gm_meta_plugin.LogHistoryContainer.reset(self)
        self.redraw_signal.emit()
